@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (c) 2014-2025 OpenRCT2 developers
+ * Copyright (c) 2014-2026 OpenRCT2 developers
  *
  * For a complete list of all authors, please refer to contributors.md
  * Interested in contributing? Visit https://github.com/OpenRCT2/OpenRCT2
@@ -19,12 +19,13 @@
 #include "Game.h"
 #include "GameState.h"
 #include "GameStateSnapshots.h"
+#include "Input.h"
 #include "OpenRCT2.h"
 #include "ParkImporter.h"
 #include "PlatformEnvironment.h"
 #include "ReplayManager.h"
 #include "Version.h"
-#include "actions/GameAction.h"
+#include "actions/GameActionRunner.h"
 #include "audio/Audio.h"
 #include "audio/AudioContext.h"
 #include "config/Config.h"
@@ -38,10 +39,13 @@
 #include "core/Path.hpp"
 #include "core/String.hpp"
 #include "core/Timer.hpp"
+#include "drawing/ColourMap.h"
+#include "drawing/Drawing.h"
 #include "drawing/IDrawingEngine.h"
 #include "drawing/Image.h"
 #include "drawing/LightFX.h"
 #include "entity/EntityTweener.h"
+#include "entity/PatrolArea.h"
 #include "interface/Chat.h"
 #include "interface/StdInOutConsole.h"
 #include "interface/Viewport.h"
@@ -72,6 +76,7 @@
 #include "ui/UiContext.h"
 #include "ui/WindowManager.h"
 #include "world/MapAnimation.h"
+#include "world/MapSelection.h"
 
 #include <chrono>
 #include <cmath>
@@ -82,10 +87,6 @@
 #include <string>
 
 using namespace OpenRCT2;
-using namespace OpenRCT2::Drawing;
-using namespace OpenRCT2::Localisation;
-using namespace OpenRCT2::Paint;
-using namespace OpenRCT2::Scripting;
 using namespace OpenRCT2::Ui;
 
 using OpenRCT2::Audio::IAudioContext;
@@ -108,7 +109,7 @@ namespace OpenRCT2
         std::unique_ptr<IUiContext> const _uiContext;
 
         // Services
-        std::unique_ptr<LocalisationService> _localisationService;
+        std::unique_ptr<Localisation::LocalisationService> _localisationService;
         std::unique_ptr<IObjectRepository> _objectRepository;
         std::unique_ptr<IObjectManager> _objectManager;
         std::unique_ptr<ITrackDesignRepository> _trackDesignRepository;
@@ -121,7 +122,7 @@ namespace OpenRCT2
 #endif
         StdInOutConsole _stdInOutConsole;
 #ifdef ENABLE_SCRIPTING
-        ScriptEngine _scriptEngine;
+        Scripting::ScriptEngine _scriptEngine;
 #endif
 #ifndef DISABLE_NETWORK
         Network::NetworkBase _network;
@@ -135,8 +136,8 @@ namespace OpenRCT2
         IScene* _activeScene = nullptr;
 
         DrawingEngine _drawingEngineType = DrawingEngine::SoftwareWithHardwareDisplay;
-        std::unique_ptr<IDrawingEngine> _drawingEngine;
-        std::unique_ptr<Painter> _painter;
+        std::unique_ptr<Drawing::IDrawingEngine> _drawingEngine;
+        std::unique_ptr<Paint::Painter> _painter;
 
         bool _initialised = false;
 
@@ -173,7 +174,7 @@ namespace OpenRCT2
             : _env(std::move(env))
             , _audioContext(std::move(audioContext))
             , _uiContext(std::move(uiContext))
-            , _localisationService(std::make_unique<LocalisationService>(*_env))
+            , _localisationService(std::make_unique<Localisation::LocalisationService>(*_env))
             , _replayManager(CreateReplayManager())
             , _gameStateSnapshots(CreateGameStateSnapshots())
 #ifdef ENABLE_SCRIPTING
@@ -182,7 +183,7 @@ namespace OpenRCT2
 #ifndef DISABLE_NETWORK
             , _network(*this)
 #endif
-            , _painter(std::make_unique<Painter>(*_uiContext))
+            , _painter(std::make_unique<Paint::Painter>(*_uiContext))
         {
             // Can't have more than one context currently.
             Guard::Assert(Instance == nullptr);
@@ -206,7 +207,7 @@ namespace OpenRCT2
             _network.Close();
 #endif
 
-            auto* windowMgr = Ui::GetWindowManager();
+            auto* windowMgr = GetWindowManager();
             windowMgr->CloseAll();
 
             // Unload objects after closing all windows, this is to overcome windows like
@@ -218,7 +219,7 @@ namespace OpenRCT2
 
             GfxObjectCheckAllImagesFreed();
             GfxUnloadCsg();
-            GfxUnloadG2AndFonts();
+            GfxUnloadG2PalettesFontsTracks();
             GfxUnloadG1();
             Audio::Close();
 
@@ -292,7 +293,7 @@ namespace OpenRCT2
             return _drawingEngineType;
         }
 
-        IDrawingEngine* GetDrawingEngine() override
+        Drawing::IDrawingEngine* GetDrawingEngine() override
         {
             return _drawingEngine.get();
         }
@@ -480,6 +481,7 @@ namespace OpenRCT2
             }
 #endif
 
+#ifndef __HAIKU__ // Haiku's user is always root, skip warning them about it.
             if (Platform::ProcessIsElevated())
             {
                 std::string elevationWarning = _localisationService->GetString(STR_ADMIN_NOT_RECOMMENDED);
@@ -492,6 +494,7 @@ namespace OpenRCT2
                     _uiContext->ShowMessageBox(elevationWarning);
                 }
             }
+#endif
 
             if (Platform::IsRunningInWine())
             {
@@ -533,9 +536,8 @@ namespace OpenRCT2
                 Drawing::LightFx::Init();
             }
 
-            ViewportInitAll();
-
             ContextInit();
+            ResetSubsystems();
 
             if (!gOpenRCT2Headless)
             {
@@ -555,6 +557,28 @@ namespace OpenRCT2
             return true;
         }
 
+        /**
+         *  rct2: 0x006E6EAC
+         */
+        void ResetSubsystems() override
+        {
+            if (!gOpenRCT2NoGraphics)
+            {
+                Drawing::initColourMaps();
+            }
+
+            WindowInitAll();
+
+            gInputFlags.clearAll();
+            InputSetState(InputState::Reset);
+            gPressedWidget.windowClassification = WindowClass::null;
+            gPickupPeepImage = ImageId();
+            ResetTooltipNotShown();
+            gMapSelectFlags.clearAll();
+            ClearPatrolAreaToRender();
+            TextinputCancel();
+        }
+
     private:
         void InitialiseRepositories()
         {
@@ -610,7 +634,7 @@ namespace OpenRCT2
         {
             assert(_drawingEngine == nullptr);
 
-            const auto initializeEngine = [&](DrawingEngine engine) -> std::unique_ptr<IDrawingEngine> {
+            const auto initializeEngine = [&](DrawingEngine engine) -> std::unique_ptr<Drawing::IDrawingEngine> {
                 try
                 {
                     auto drawingEngineFactory = _uiContext->GetDrawingEngineFactory();
@@ -705,7 +729,7 @@ namespace OpenRCT2
             if (!gOpenRCT2Headless && isMainThread)
             {
                 _uiContext->ProcessMessages();
-                auto* windowMgr = Ui::GetWindowManager();
+                auto* windowMgr = GetWindowManager();
                 windowMgr->InvalidateByClass(WindowClass::progressWindow);
                 Draw();
             }
@@ -1035,7 +1059,7 @@ namespace OpenRCT2
             {
                 return false;
             }
-            GfxLoadG2FontsAndTracks();
+            GfxLoadG2PalettesFontsTracks();
             GfxLoadCsg();
             FontSpriteInitialiseCharacters();
             return true;
@@ -1215,7 +1239,7 @@ namespace OpenRCT2
 
             if (!gOpenRCT2Headless)
             {
-                _preloaderScene->SetOnComplete([&]() { SwitchToStartUpScene(); });
+                GetPreloaderScene()->SetOnComplete([&]() { SwitchToStartUpScene(); });
             }
             else
             {
@@ -1299,6 +1323,8 @@ namespace OpenRCT2
 
             UpdateTimeAccumulators(deltaTime);
 
+            Network::Update();
+
             if (useVariableFrame)
             {
                 RunVariableFrame(deltaTime);
@@ -1307,6 +1333,8 @@ namespace OpenRCT2
             {
                 RunFixedFrame(deltaTime);
             }
+
+            Network::Flush();
         }
 
         void UpdateTimeAccumulators(float deltaTime)
@@ -1332,7 +1360,7 @@ namespace OpenRCT2
 
             if (_ticksAccumulator < kGameUpdateTimeMS)
             {
-                const auto sleepTimeSec = (kGameUpdateTimeMS - _ticksAccumulator);
+                const auto sleepTimeSec = std::min(kNetworkUpdateTimeMS, kGameUpdateTimeMS - _ticksAccumulator);
                 Platform::Sleep(static_cast<uint32_t>(sleepTimeSec * 1000.f));
                 return;
             }
@@ -1580,7 +1608,7 @@ namespace OpenRCT2
     }
 
     std::unique_ptr<IContext> CreateContext(
-        std::unique_ptr<IPlatformEnvironment>&& env, std::unique_ptr<Audio::IAudioContext>&& audioContext,
+        std::unique_ptr<IPlatformEnvironment>&& env, std::unique_ptr<IAudioContext>&& audioContext,
         std::unique_ptr<IUiContext>&& uiContext)
     {
         return std::make_unique<Context>(std::move(env), std::move(audioContext), std::move(uiContext));
@@ -1596,6 +1624,11 @@ namespace OpenRCT2
         GetWindowManager()->Init();
     }
 
+    void ContextResetSubsystems()
+    {
+        GetContext()->ResetSubsystems();
+    }
+
     bool ContextLoadParkFromStream(void* stream)
     {
         return GetContext()->LoadParkFromStream(static_cast<IStream*>(stream), "");
@@ -1706,55 +1739,55 @@ namespace OpenRCT2
 
     WindowBase* ContextOpenWindow(WindowClass wc)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         return windowManager->OpenWindow(wc);
     }
 
     WindowBase* ContextOpenWindowView(WindowView view)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         return windowManager->openView(view);
     }
 
     WindowBase* ContextOpenDetailWindow(WindowDetail type, int32_t id)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         return windowManager->openDetails(type, id);
     }
 
     WindowBase* ContextOpenIntent(Intent* intent)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         return windowManager->OpenIntent(intent);
     }
 
     void ContextBroadcastIntent(Intent* intent)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         windowManager->BroadcastIntent(*intent);
     }
 
     void ContextForceCloseWindowByClass(WindowClass windowClass)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         windowManager->ForceClose(windowClass);
     }
 
     WindowBase* ContextShowError(StringId title, StringId message, const Formatter& args, const bool autoClose /* = false */)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         return windowManager->ShowError(title, message, args, autoClose);
     }
 
     void ContextHandleInput()
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         windowManager->HandleInput();
     }
 
     void ContextInputHandleKeyboard(bool isTitle)
     {
-        auto windowManager = Ui::GetWindowManager();
+        auto windowManager = GetWindowManager();
         windowManager->HandleKeyboard(isTitle);
     }
 
@@ -1763,7 +1796,7 @@ namespace OpenRCT2
         GetContext()->Quit();
     }
 
-    u8string ContextOpenCommonFileDialog(Ui::FileDialogDesc& desc)
+    u8string ContextOpenCommonFileDialog(FileDialogDesc& desc)
     {
         try
         {
